/**
 * Represents a classic stack-based type. A stack is last-in-first-out (LIFO),
 * so you can only ever add or remove elements from the top of the stack
 * (the most recently added element).
 *
 * To ensure the stack uses contiguous memory, this data type uses an
 * array internally.
 */
export default class Stack<T> implements Iterable<T> {
    private readonly array;
    /**
     * Constructs a new Stack.
     *
     * @param elements Initial elements to add to the Stack.
     *   Note that these are added in-sequence, i.e. the first element
     *   is the one at the *bottom*, not the top.
     */
    constructor(...elements: T[]);
    [Symbol.iterator](): Iterator<T>;
    /**
     * Adds an element to the top of the Stack.
     */
    push(element: T): void;
    /**
     * Removes the topmost element from the stack and returns it.
     *
     * This function will throw an error if the Stack is empty.
     */
    pop(): T;
    /** Gets the number of elements in the Stack. */
    get size(): number;
    /**
     * Returns the topmost element without removing it from the Stack.
     * If the Stack is empty, returns `undefined`.
     */
    peek(): T | undefined;
    /** Returns true if the Stack is empty. */
    isEmpty(): boolean;
    /** Returns an array representation of this Stack. */
    toJSON(): T[];
}
