/**
 * A convenience class allowing bitwise operations on a flags value.
 * This class also contains static methods that can be used without
 * constructing an instance.
 */
export default class Flags {
    private value;
    constructor(initialValue?: number);
    /** Checks if the number includes the given flag. */
    has(flag: number): boolean;
    /** Checks if the number includes any of the given flags. */
    hasSome(...flags: number[]): boolean;
    /** Checks if the number includes all of the given flags. */
    hasEvery(...flags: number[]): boolean;
    /** Sets the given flags. */
    set(...flags: number[]): this;
    /** Unsets the given flags. */
    unset(...flags: number[]): this;
    /** Clears all flags. */
    clear(): this;
    /** Checks if the number includes the given flag. */
    static has(flags: number, flag: number): boolean;
    /** Checks if the number includes any of the given flags. */
    static hasSome(flags: number, ...someFlags: number[]): boolean;
    /** Checks if the number includes all of the given flags. */
    static hasEvery(flags: number, ...everyFlag: number[]): boolean;
    /** Returns a new number with the given flags set on `flags`. */
    static set(flags: number, ...flagsToSet: number[]): number;
    /** Returns a new number with the given flags unset on `flags`. */
    static unset(flags: number, ...flagsToUnset: number[]): number;
    /** Converts the flags to a number. */
    toNumber(): number;
    /** Converts the flags to a number. */
    toJSON(): number;
    /** Converts the flags to a number string. */
    toString(): string;
}
