import { fluentIs, fluentIsLazy, set as setIs } from './is.js';
export function is(...objects) {
    setIs(objects);
    if (objects.length === 0) {
        return fluentIsLazy;
    }
    return fluentIs;
}
/**
 * Maps one or multiple objects to a result.
 *
 * This function is useful to compress multiple lines into one, especially
 * for results based on multiple or repeated values, e.g.:
 *
 * @example
 * // get penultimate element of array
 * // would otherwise require you to store myArray
 * // in a local variable
 * map(getMyArray(), array => array[array.length - 2]);
 */
export function map(...args) {
    const callback = args[args.length - 1];
    return callback(...args.slice(0, -1));
}
