import { every, any, approximately, array, falsy, float, func, instanceOf, integer, keyOf, notNull, nullish, oneOf, truthy, type as isType, value as isValue, arrayOf, value } from './matchers.js';
/** Whether every object must meet the condition. */
let _every = true;
let not = false;
let objects = [];
function evaluate(objects, inverted, every, condition) {
    const result = every
        ? objects.every(o => condition(o))
        : objects.some(o => condition(o));
    return inverted ? !result : result;
}
function evaluateLazy(inverted, every, condition) {
    return (...objects) => evaluate(objects, inverted, every, condition);
}
function makeFluentIs(evaluator) {
    return {
        any() {
            _every = false;
            return this;
        },
        every() {
            _every = true;
            return this;
        },
        get not() {
            not = true;
            return this;
        },
        oneOf(...these) {
            return evaluator(oneOf(...these.map(x => value(x))));
        },
        truthy() {
            return evaluator(truthy());
        },
        falsy() {
            return evaluator(falsy());
        },
        matching(...matchers) {
            return evaluator(every(...matchers));
        },
        matchingSome(...matchers) {
            return evaluator(any(...matchers));
        },
        keyOf(object) {
            return evaluator(keyOf(object));
        },
        arrayOf(object) {
            return evaluator(arrayOf(value(object)));
        },
        type(type) {
            return evaluator(isType(type));
        },
        instanceOf(constructor) {
            return evaluator(instanceOf(constructor));
        },
        value(value) {
            return evaluator(isValue(value));
        },
        notNull() {
            return evaluator(notNull());
        },
        nullish() {
            return evaluator(nullish());
        },
        integer() {
            return evaluator(integer());
        },
        float() {
            return evaluator(float());
        },
        approximately(comparedValue, tolerance) {
            return evaluator(approximately(comparedValue, tolerance));
        },
        array() {
            return evaluator(array());
        },
        func(argumentCount) {
            return evaluator(func(argumentCount));
        }
    };
}
/** @internal */
export const fluentIs = makeFluentIs(matcher => evaluate(objects, not, _every, matcher));
/** @internal */
export const fluentIsLazy = makeFluentIs(matcher => evaluateLazy(not, _every, matcher));
/** @internal */
export function set(objs) {
    _every = true;
    not = false;
    objects = objs;
}
