import { ArrayType, Typeof, TypeofResult, Constructor, Falsy, Func, Predicate, Truthy } from '../types.js';
/** Matches a truthy value. */
export declare function truthy<T>(): (item: T) => item is Truthy<T>;
/** Matches a falsy value. */
export declare function falsy<T>(): (item: T) => item is Falsy<T>;
/** Matches if any of the matchers match. */
export declare function oneOf<T extends Array<(item: unknown) => boolean>>(...matchers: T): (item: unknown) => item is ArrayType<T>;
/** Matches if the passed object contains the value as a key. */
export declare function keyOf<T extends object>(object: T): (item: unknown) => item is keyof T;
/** Matches a value of the specified type. */
export declare function type<T extends TypeofResult>(type: T): (item: unknown) => item is Typeof<T>;
/** Matches a value of the specified instance type or any inherited types. */
export declare function instanceOf<T extends Constructor>(constructor: T): (item: unknown) => item is T;
/** Matches a value by referential equality. */
export declare function value<T>(value: T): (item: unknown) => item is T;
/** Matches a value that is not null or undefined. */
export declare function notNull<T>(): (item: T) => item is NonNullable<T>;
/** Matches a value that is null or undefined. */
export declare function nullish(): (item: unknown) => item is null | undefined;
/** Matches a string. */
export declare function string(): (item: unknown) => item is string;
/** Matches a number. */
export declare function number(): (item: unknown) => item is number;
/** Matches an integral number. */
export declare function integer(): (item: unknown) => item is number;
/** Matches a floating point number (but not integral numbers). */
export declare function float(): (item: unknown) => item is number;
/** Matches a number that approximately equals the passed number. */
export declare function approximately(comparedValue: number, tolerance?: number): (item: unknown) => item is number;
/** Matches an array. */
export declare function array(): (item: unknown) => item is unknown[];
/** Matches an array where each element matches the given matcher. */
export declare function arrayOf<T>(matcher: (item: unknown) => item is T): (item: unknown) => item is T[];
/** Matches a function. */
export declare function func(): (item: unknown) => item is Func;
/** Matches a function with the given number of formal parameters. */
export declare function func<N extends number>(argumentCount: N): (item: unknown) => item is Func<N>;
/** Matches if all passed matchers match. */
export declare function every<T = unknown>(...matchers: Predicate<T>[]): (item: T) => boolean;
/** Matches if any of the passed matchers match. */
export declare function any<T = unknown>(...matchers: Predicate<T>[]): (item: T) => boolean;
