import type ReadonlyRange from './ReadonlyRange.js';
/**
 * Represents a range between two numbers with a variety of functions
 * that simplify working with number ranges.
 *
 * Note that all property methods are cached calculated properties.
 * That is, calculation is deferred until the method is first called
 * and then cached for subsequent calls unless the `from` or `to`
 * values are modified.
 */
export default class Range implements ReadonlyRange {
    private _from;
    private _to;
    private _min?;
    private _max?;
    private _center?;
    private _span?;
    constructor();
    constructor(from: number, to: number);
    get from(): number;
    get to(): number;
    /**
     * Sets this range to the new values and returns itself.
     *
     * Note that `from` does not need to be smaller than `to`. Using an inverted
     * range can be helpful for, for instance, `Range.at()`.
     */
    set(from: number, to: number): this;
    /** Inverts the range so that `from` becomes `to` and `to` becomes `from`. */
    invert(): this;
    private reset;
    min(): number;
    max(): number;
    center(): number;
    span(): number;
    clamp(value: number): number;
    contains(range: ReadonlyRange): boolean;
    contains(value: number, tolerance?: number): boolean;
    isBetween(value: number): boolean;
    overlaps(range: ReadonlyRange): boolean;
    intersectionPoint(range: ReadonlyRange): number;
    intersect(range: ReadonlyRange): Range;
    union(range: ReadonlyRange): Range;
    getOffset(range: ReadonlyRange): number;
    at(value: number): number;
    relative(value: number): number;
    wrap(value: number): number;
    isEmpty(): boolean;
    equals(range: ReadonlyRange): boolean;
    equals(from: number, to: number): boolean;
    clone(): Range;
}
