import { hasFunction, isPrimitive } from './typechecking.js';
const DEFAULT_OPTIONS = {};
function truncateAfter(array, map, limit) {
    const strings = [];
    const length = limit != null && limit > array.length ? array.length : (limit !== null && limit !== void 0 ? limit : array.length);
    for (let i = 0; i < length; i++) {
        strings.push(map(array[i]));
    }
    if (strings.length < array.length) {
        strings.push('...');
    }
    return strings;
}
/**
 * The symbol used to implement custom `Objects.stringify()` behaviour
 * for objects. See the {@link Stringifiable} interface.
 */
export const Stringify = Symbol('stringify');
Object.defineProperty(Object.prototype, Stringify, { value: function (options) {
        var _a;
        const constructorName = typeof this === 'object' && this.constructor != null ? this.constructor.name : 'Object';
        if (options.typesOnly) {
            return constructorName;
        }
        let result = '';
        if (!options.hideClasses && constructorName !== 'Object' && constructorName !== 'Array') {
            result += constructorName + ' ';
        }
        if (((_a = options.useToString) !== null && _a !== void 0 ? _a : true) && this != null && typeof this['toString'] === 'function') {
            const string = this.toString();
            if (!(string.startsWith('[object') && string.endsWith(']'))) {
                if (result.length === 0) {
                    return string;
                }
                return result.concat(`{ ${string} }`);
            }
        }
        const entries = Object.entries(this);
        if (entries.length === 0) {
            return '{}';
        }
        if (options.truncateContents === true) {
            return result.concat('{ ... }');
        }
        const stringifiedElements = truncateAfter(entries, element => `${element[0]}: ${stringify(element[1], options)}`, typeof options.truncateContents === 'number' ? options.truncateContents : undefined);
        return result.concat(`{ ${stringifiedElements.join(', ')} }`);
    } });
// eslint-disable-next-line @typescript-eslint/ban-types
Object.defineProperty(Function.prototype, Stringify, { value: function (options) {
        if (options.typesOnly) {
            return 'function';
        }
        const string = this.toString();
        const openingBraceIndex = string.indexOf('{');
        const closingBraceIndex = openingBraceIndex === -1 ? -1 : string.indexOf('}');
        if (openingBraceIndex === -1 || closingBraceIndex === -1 || !string.includes('\n')) {
            return string;
        }
        return string.substring(0, openingBraceIndex) + '{ ... }';
    } });
Object.defineProperty(Array.prototype, Stringify, { value: function (options) {
        if (options.typesOnly) {
            const types = new Set(this.map(x => stringify(x, options)));
            if (types.size === 0) {
                types.add('unknown');
            }
            if (types.size === 1) {
                return `${[...types][0]}[]`;
            }
            return `(${[...types].join(' | ')})[]`;
        }
        const prefix = this.constructor !== Array && !options.hideClasses ? this.constructor.name : '';
        if (options.truncateContents === true && this.length !== 0) {
            return `${prefix}[...]`;
        }
        const stringifiedElements = truncateAfter(this, element => stringify(element, options), typeof options.truncateContents === 'number' ? options.truncateContents : undefined);
        return `${prefix}[${stringifiedElements.join(', ')}]`;
    } });
/**
 * Converts an arbitrary value to string. See `StringifyOptions` for
 * information on how to customize this function.
 *
 * This function was primarily designed for the purposes of readable
 * and expressive debug output (for instance, the `assert` module
 * heavily utilizes it), but it can be used for other purposes as well.
 *
 * If a type implements {@link Stringifiable} (which utilizes the
 * {@link Stringify} symbol), it will be used to convert that type
 * to a string representation here.
 */
export function stringify(value, options = DEFAULT_OPTIONS) {
    var _a;
    const replacerResult = (_a = options.replacer) === null || _a === void 0 ? void 0 : _a.call(options, value);
    if (replacerResult != null) {
        return replacerResult;
    }
    if (value === null) {
        return 'null';
    }
    if (value === undefined) {
        return 'undefined';
    }
    if (isPrimitive(value)) {
        if (options.typesOnly || options.primitiveTypesOnly) {
            return typeof value;
        }
        if (typeof value === 'string') {
            return options.omitQuotes ? value : `"${value}"`;
        }
        return value.toString();
    }
    if (hasFunction(value, Stringify)) {
        return value[Stringify](options);
    }
    return JSON.stringify(value);
}
