import { isObject, isObjectLiteral } from './typechecking.js';
function match(expected, actual) {
    if (typeof expected === 'function') {
        return expected(actual);
    }
    if (isObjectLiteral(expected)) {
        if (!isObject(actual)) {
            return false;
        }
        return structure(actual, expected);
    }
    throw new Error(`Invalid type for 'expected': must be a predicate or a nested Structure but was: ${expected}`);
}
/**
 * Checks if the passed object conforms to the given structure.
 *
 * @param exhaustive If true, the check fails if {@link object} contains more
 * properties than {@link struct} defines. Otherwise superfluous properties
 * are simply ignored.
 */
export function structure(object, struct, exhaustive = false) {
    if (!isObject(object)) {
        return false;
    }
    if (!isObject(struct)) {
        throw new Error('Objects.structure(): struct must be an object!');
    }
    if (exhaustive && Object.keys(object).length > Object.keys(struct).length) {
        // If object has more props than struct, always fail.
        // If struct has more props than object, fail only if that property is
        // not defined as 'undefined' in struct (see match()).
        return false;
    }
    for (const property in struct) {
        const expected = struct[property];
        const actual = object[property];
        if (!match(expected, actual)) {
            return false;
        }
    }
    return true;
}
