/** Returns true if the string contains no characters. */
export declare function isEmpty(string: string): boolean;
/** Returns true if the string is empty or contains only whitespace. */
export declare function isWhitespace(string: string): boolean;
/**
 * Splits the string at the given indexes, returning a new array with the
 * newly formed substrings.
 *
 * Note that surrogate pairs are not broken apart. Duplicate indexes
 * will be ignored, ensuring that the resulting array never contains
 * a zero-length segment.
 *
 * @example
 * Strings.splitAt('bananas', 3, 5);  // -> ['ban', 'an', 'as']
 */
export declare function splitAt(string: string, ...indexes: number[]): string[];
/**
 * Replaces a segment of the string by cutting out a segment of the string
 * defined by `index` and `length`, and inserting `replaceWith` in its place
 * (if specified).
 */
export declare function splice(string: string, index: number, length: number, replaceWith?: string): string;
/**
 * Removes all of the given substrings from the string.
 */
export declare function strip(from: string, ...what: string[]): string;
/** Removes all of the given substrings from the start of the string. */
export declare function stripStart(from: string, ...what: string[]): string;
/** Removes all of the given substrings from the end of the string. */
export declare function stripEnd(from: string, ...what: string[]): string;
/** Converts a string to camelCase from either snake_case, ALL_CAPS, hyphenated-case, or spaced text. */
export declare function camelCase(source: string): string;
/** Converts a string to pascalCase from either snake_case, ALL_CAPS, hyphenated-case, or spaced text. */
export declare function pascalCase(source: string): string;
/** Prefixes the string with the specified prefix only if the string is not already prefixed with it. */
export declare function prefix(string: string, prefix: string): string;
/** Suffixes the string with the specified prefix only if the string is not already suffixed with it. */
export declare function suffix(string: string, suffix: string): string;
/** Tests if the given string is a valid URL. */
export declare function isUrl(url: string): boolean;
/**
 * Capitalizes the first letter of the string.
 * Has no effect if the first character isn't a letter.
 */
export declare function capitalize(string: string): string;
/**
 * If the first letter of the string is capitalized, uncapitalizes it.
 * Has no effect if the first character isn't a letter.
 */
export declare function uncapitalize(string: string): string;
/**
 * Returns the first {@link count} characters in the string.
 *
 * If {@link string} is shorter than {@link count}, returns the entire string.
 *
 * If {@link count} is negative, takes from the end of the string instead.
 */
export declare function take(string: string, count: number): string;
/**
 * Returns the last {@link count} characters in the string.
 *
 * If {@link string} is shorter than {@link count}, returns the entire string.
 *
 * If {@link count} is negative, takes from the start of the string instead.
 */
export declare function takeLast(string: string, count: number): string;
/**
 * Skips the first {@link count} characters in the string and returns the rest.
 *
 * If {@link string} is shorter than {@link count}, returns an empty string.
 *
 * If {@link count} is negative, skips from the end of the string instead.
 */
export declare function skip(string: string, count: number): string;
/**
 * Skips the last {@link count} characters in the string and returns the rest.
 *
 * If {@link string} is shorter than {@link count}, returns an empty string.
 *
 * If {@link count} is negative, skips from the start of the string instead.
 */
export declare function skipLast(string: string, count: number): string;
/**
 * Returns all characters in the string before the first occurrence of {@link substring}.
 *
 * If {@link substring} is not part of the string, returns the entire string.
 */
export declare function takeUntil(string: string, substring: string): string;
/**
 * Returns all characters in the string before the first occurrence of {@link substring},
 * including {@link substring} itself.
 *
 * If {@link substring} is not part of the string, returns the entire string.
 */
export declare function takeUntilAfter(string: string, substring: string): string;
/**
 * Returns all characters in the string before the last occurrence of {@link substring}.
 *
 * If {@link substring} is not part of the string, returns the entire string.
 */
export declare function takeUntilLast(string: string, substring: string): string;
/**
 * Returns all characters in the string before the last occurrence of {@link substring},
 * including {@link substring} itself.
 *
 * If {@link substring} is not part of the string, returns the entire string.
 */
export declare function takeUntilAfterLast(string: string, substring: string): string;
/**
 * Returns all characters in the string after the first occurrence of {@link substring},
 * including {@link substring} itself.
 *
 * If {@link substring} is not part of the string, returns an empty string.
 */
export declare function skipUntil(string: string, substring: string): string;
/**
 * Returns all characters in the string after the first occurrence of {@link substring}.
 *
 * If {@link substring} is not part of the string, returns an empty string.
 */
export declare function skipUntilAfter(string: string, substring: string): string;
/**
 * Returns all characters in the string after the last occurrence of {@link substring},
 * including {@link substring} itself.
 *
 * If {@link substring} is not part of the string, returns an empty string.
 */
export declare function skipUntilLast(string: string, substring: string): string;
/**
 * Returns all characters in the string after the last occurrence of {@link substring}.
 *
 * If {@link substring} is not part of the string, returns an empty string.
 */
export declare function skipUntilAfterLast(string: string, substring: string): string;
